@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2020 the orix developers
+# Copyright 2018-2021 the orix developers
 #
 # This file is part of orix.
 #
@@ -18,6 +18,7 @@
 
 import copy
 
+import matplotlib.pyplot as plt
 import numpy as np
 
 from orix.crystal_map.crystal_map_properties import CrystalMapProperties
@@ -160,9 +161,14 @@ class CrystalMap:
             phase_ids = phase_list.ids
             n_different = len(phase_ids) - len(unique_phase_ids)
             if n_different > 0:
-                # Remove superfluous phases
-                for i in phase_list.ids[::-1][:n_different]:
-                    del phase_list[i]
+                # Remove superfluous phases by removing the phases whose
+                # ID is not in the ID array, in descending list order
+                for i in phase_ids[::-1]:
+                    if i not in unique_phase_ids:
+                        del phase_list[i]
+                        n_different -= 1
+                    if n_different == 0:
+                        break
             elif n_different < 0:
                 # Create new phase list adding the missing phases with
                 # default initial values
@@ -491,6 +497,9 @@ class CrystalMap:
         Properties: iq, dp
         Scan unit: um
         """
+        # TODO: Crop new map to the extremal spatial values (e.g. if all values in first
+        #  or last row/column are masked out by the key), i.e. not just mask the values
+
         # Initiate a mask to be added to the returned copy of the
         # CrystalMap object, to ensure that only the unmasked values are
         # in the data of the copy (True in `is_in_data`). First, no points
@@ -570,7 +579,7 @@ class CrystalMap:
         unique_phases = np.unique(phase_ids)
         p_sizes = [np.where(phase_ids == i)[0].size for i in unique_phases]
         id_len = 5
-        ori_len = max(max([len(str(p_size)) for p_size in p_sizes]) + 8, 12)
+        ori_len = max(max([len(str(p_size)) for p_size in p_sizes]) + 9, 12)
         name_len = max(max([len(n) for n in names]), 4)
         sg_len = max(max([len(i) for i in sg_names]), 11)
         pg_len = max(max([len(i) for i in pg_names]), 11)
@@ -619,6 +628,44 @@ class CrystalMap:
 
         return representation
 
+    def deepcopy(self):
+        """Return a deep copy using :func:`copy.deepcopy` function."""
+        return copy.deepcopy(self)
+
+    @classmethod
+    def empty(cls, shape=None, step_sizes=None):
+        """Create a crystal map of a given shape and step sizes with
+        identity rotations.
+
+        Parameters
+        ----------
+        shape : tuple of int, optional
+            Map shape. Default is a 2D map of shape (5, 10), i.e. with
+            five rows and ten columns.
+        step_sizes : tuple, optional
+            Map step sizes. If None (default), it is set to 1 px in each
+            map direction given by `shape`.
+
+        Returns
+        -------
+        CrystalMap
+
+
+        Examples
+        --------
+        >>> from orix.crystal_map import CrystalMap
+        >>> xmap = CrystalMap.empty((5, 10))
+        >>> xmap
+        Phase  Orientations  Name  Space group  Point group  Proper point group     Color
+            0   50 (100.0%)  None         None         None                None  tab:blue
+        Properties:
+        Scan unit: px
+        >>> xmap.plot("x")  # Increasing towards the right
+        """
+        d, n = create_coordinate_arrays(shape, step_sizes)
+        d["rotations"] = Rotation.identity((n,))
+        return cls(**d)
+
     def get_map_data(self, item, decimals=3, fill_value=None):
         """Return an array of a class instance attribute, with values
         equal to ``False`` in ``self.is_in_data`` set to `fill_value`, of
@@ -636,7 +683,7 @@ class CrystalMap:
             How many decimals to round data point values to (default is
             3).
         fill_value : None, optional
-            Value to fill points not in the data with. If ``None``
+            Value to fill points not in the data with. If None
             (default), np.nan is used.
 
         Returns
@@ -655,17 +702,22 @@ class CrystalMap:
         # TODO: Better account for `item.shape`, e.g. quaternions
         #  (item.shape[-1] == 4) in a more general way than here (not more
         #  if/else)!
-        array = np.zeros(np.prod(map_shape))
+        map_size = np.prod(map_shape)
         if isinstance(item, np.ndarray):
+            array = np.empty(map_size, dtype=item.dtype)
             if item.shape[-1] == 3:  # Assume RGB
                 map_shape += (3,)
                 array = np.column_stack((array,) * 3)
         elif item in ["orientations", "rotations"]:  # Definitely RGB
+            array = np.empty(map_size, dtype=np.float64)
             map_shape += (3,)
             array = np.column_stack((array,) * 3)
+        else:
+            array = np.empty(map_size, dtype=np.float64)
 
         # Enter non-masked values into array
         if isinstance(item, np.ndarray):
+            # TODO: Account for 2D map with more than one value per point
             array[self.is_in_data] = item
         elif item in ["orientations", "rotations"]:
             if item == "rotations":
@@ -688,30 +740,138 @@ class CrystalMap:
             if data is None:
                 raise ValueError(f"{item} is {data}.")
             else:
+                # TODO: Account for 2D map with more than one value per point
                 array[self.is_in_data] = data
-
-        # Round values
-        rounded_array = np.round(array, decimals=decimals)
+                array = array.astype(data.dtype)
 
         # Slice and reshape array
         slices = self._data_slices_from_coordinates()
-        reshaped_array = rounded_array.reshape(map_shape)
-        output_array = reshaped_array[slices]
+        reshaped_array = array.reshape(map_shape)
+        sliced_array = reshaped_array[slices]
 
-        # Reshape and slice mask with points *not* in data
+        # Reshape and slice mask with points not in data
         if array.shape[-1] == 3:  # RGB
             not_in_data = np.dstack((~self.is_in_data,) * 3)
         else:  # Scalar
             not_in_data = ~self.is_in_data
         not_in_data = not_in_data.reshape(map_shape)[slices]
 
-        output_array[not_in_data] = fill_value
+        # Fill points not in data with the fill value
+        if not_in_data.any():
+            if fill_value is None or fill_value is np.nan:
+                sliced_array = sliced_array.astype(np.float64)
+            sliced_array[not_in_data] = fill_value
+
+        # Round values
+        if np.issubdtype(array.dtype, np.bool_):
+            output_array = sliced_array
+        else:
+            output_array = np.round(sliced_array, decimals=decimals)
 
         return output_array
 
-    def deepcopy(self):
-        """Return a deep copy using :func:`copy.deepcopy` function."""
-        return copy.deepcopy(self)
+    def plot(
+        self,
+        value=None,
+        overlay=None,
+        scalebar=True,
+        scalebar_properties=None,
+        legend=True,
+        legend_properties=None,
+        colorbar=False,
+        colorbar_label=None,
+        colorbar_properties=dict(),
+        remove_padding=False,
+        return_figure=False,
+        figure_kwargs=dict(),
+        **kwargs,
+    ):
+        r"""Plot a 2D map with any crystallographic map property as map
+        values.
+
+        Wraps :meth:`matplotlib.axes.Axes.imshow`, see that method for
+        relevant keyword arguments.
+
+        Parameters
+        ----------
+        value : numpy.ndarray, optional
+            Attribute array to plot. If None (default), a phase map is
+            plotted.
+        overlay : str or numpy.ndarray, optional
+            Name of map property or a property array to use in the
+            alpha (RGBA) channel. The property range is adjusted for
+            maximum contrast. Default is None.
+        scalebar : bool, optional
+            Whether to add a scalebar (default is True) along the
+            horizontal map dimension.
+        scalebar_properties : dict, optional
+            Keyword arguments passed to
+            :class:`matplotlib_scalebar.scalebar.ScaleBar`.
+        legend : bool, optional
+            Whether to add a legend to the plot. This is only
+            implemented for a phase plot (in which case default is
+            True).
+        legend_properties : dict, optional
+            Keyword arguments passed to :meth:`matplotlib.axes.legend`.
+        colorbar : bool, optional
+            Whether to add an opinionated colorbar (default is False).
+        colorbar_label : str, optional
+            Label/title of colorbar.
+        colorbar_properties : dict, optional
+            Keyword arguments passed to
+            :meth:`orix.plot.CrystalMapPlot.add_colorbar`.
+        axes : tuple of ints, optional
+            Which data axes to plot if data has more than two
+            dimensions. The index of data to plot in the final dimension
+            is determined by `depth`. If None (default), data along the
+            two last axes is plotted.
+        depth : int, optional
+            Which layer along the third axis to plot if data has more
+            than two dimensions. If None (default), data in the first
+            index (layer) is plotted.
+        return_figure: bool, optional
+            Whether to return the figure (default is False).
+        figure_kwargs : dict, optional
+            Keyword arguments passed to
+            :func:`matplotlib.pyplot.subplots`.
+        kwargs
+            Keyword arguments passed to
+            :meth:`matplotlib.axes.Axes.imshow`.
+
+        Returns
+        -------
+        fig : matplotlib.figure.Figure
+            The created figure, returned if `return_figure` is True.
+
+        See Also
+        --------
+        matplotlib.axes.Axes.imshow
+        orix.plot.CrystalMapPlot.plot_map
+        orix.plot.CrystalMapPlot.add_scalebar
+        orix.plot.CrystalMapPlot.add_overlay
+        orix.plot.CrystalMapPlot.add_colorbar
+        """
+        # Register "plot_map" projection with Matplotlib
+        import orix.plot.crystal_map_plot
+
+        fig, ax = plt.subplots(subplot_kw=dict(projection="plot_map"), **figure_kwargs)
+        ax.plot_map(
+            self,
+            value=value,
+            scalebar=scalebar,
+            scalebar_properties=scalebar_properties,
+            legend=legend,
+            legend_properties=legend_properties,
+            **kwargs,
+        )
+        if overlay is not None:
+            ax.add_overlay(self, overlay)
+        if remove_padding:
+            ax.remove_padding()
+        if colorbar:
+            ax.add_colorbar(label=colorbar_label, **colorbar_properties)
+        if return_figure:
+            return fig
 
     @staticmethod
     def _step_size_from_coordinates(coordinates):
@@ -768,3 +928,50 @@ class CrystalMap:
         for dim_slice in self._data_slices_from_coordinates():
             data_shape.append(dim_slice.stop - dim_slice.start)
         return tuple(data_shape)
+
+
+def create_coordinate_arrays(shape=None, step_sizes=None):
+    """Create flattened coordinate arrays from a given map shape and
+    step sizes, suitable for initializing a
+    :class:`orix.crystal_map.CrystalMap`. Arrays for 1D, 2D, or 3D maps
+    can be returned.
+
+    Parameters
+    ----------
+    shape : tuple of int, optional
+        Map shape. Default is a 2D map of shape (5, 10) with five rows
+        and ten columns. Can be up to 3D.
+    step_sizes : tuple, optional
+        Map step sizes. If None (default), it is set to 1 px in each
+        map direction given by `shape`.
+
+    Returns
+    -------
+    d : dict of numpy.ndarray
+        Dictionary with keys "z", "y", and "x", depending on the length
+        of `shape`, with coordinate arrays.
+    map_size : int
+        Number of map points.
+    """
+    if shape is None:
+        shape = (5, 10)
+    ndim = len(shape)
+    if step_sizes is None:
+        step_sizes = (1,) * ndim
+
+    # Set up as if a 3D map is to be returned
+    dz, dy, dx = (1,) * (3 - ndim) + step_sizes
+    nz, ny, nx = (1,) * (3 - ndim) + shape
+    d = dict()
+
+    # Add coordinate arrays depending on the number of map dimensions
+    d["x"] = np.tile(np.arange(nx) * dx, ny * nz).flatten()
+    map_size = nx
+    if ndim > 1:
+        d["y"] = np.tile(np.sort(np.tile(np.arange(ny) * dy, nx)), nz).flatten()
+        map_size *= ny
+    if ndim > 2:
+        d["z"] = np.array([np.ones(ny * nx) * i * dz for i in range(nz)]).flatten()
+        map_size *= nz
+
+    return d, map_size
