@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2020 the orix developers
+# Copyright 2018-2021 the orix developers
 #
 # This file is part of orix.
 #
@@ -181,7 +181,7 @@ class Phase:
                     value = correct
                     break
             for point_group in _groups:
-                if value.replace("-", "") == point_group.name.replace("-", ""):
+                if value == point_group.name:
                     value = point_group
                     break
         if not isinstance(value, Symmetry) and value is not None:
@@ -201,6 +201,12 @@ class Phase:
                     self.space_group = None
             self._point_group = value
 
+    @property
+    def is_hexagonal(self):
+        """Whether the crystal structure is hexagonal/trigonal or not.
+        """
+        return np.allclose(self.structure.lattice.abcABG()[3:], [90, 90, 120])
+
     def __repr__(self):
         if self.point_group is not None:
             pg_name = self.point_group.name
