@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2020 the orix developers
+# Copyright 2018-2021 the orix developers
 #
 # This file is part of orix.
 #
@@ -19,13 +19,15 @@
 import pytest
 import numpy as np
 
-from orix.io import load, loadang
+from orix.crystal_map import CrystalMap, Phase
+from orix.io import load, loadang, save
 from orix.io.plugins.ang import (
     _get_header,
     _get_phases_from_header,
     _get_vendor_columns,
+    _get_nrows_ncols_step_sizes,
+    _get_column_width,
 )
-
 from orix.tests.conftest import (
     ANGFILE_TSL_HEADER,
     ANGFILE_ASTAR_HEADER,
@@ -79,9 +81,9 @@ def test_loadang(angfile_astar, expected_data):
     assert np.allclose(loaded_data.data, expected_data)
 
 
-class TestAngPlugin:
+class TestAngReader:
     @pytest.mark.parametrize(
-        "angfile_tsl, map_shape, step_sizes, phase_id, n_unknown_columns, example_rot",
+        "angfile_tsl, map_shape, step_sizes, phase_id, example_rot",
         [
             (
                 # Read by angfile_tsl() via request.param (passed via `indirect` below)
@@ -97,7 +99,6 @@ class TestAngPlugin:
                 (5, 3),
                 (0.1, 0.1),
                 np.zeros(5 * 3, dtype=int),
-                5,
                 np.array(
                     [[1.59942, 2.37748, -1.74690], [1.59331, 2.37417, -1.74899]]
                 ),  # rotations as rows of Euler angle triplets
@@ -115,7 +116,6 @@ class TestAngPlugin:
                 (8, 4),
                 (1.5, 1.5),
                 np.zeros(8 * 4, dtype=int),
-                5,
                 np.array(
                     [[-0.12113, 2.34188, 1.31702], [-0.47211, 0.79936, -1.80973]]
                 ),  # rotations as rows of Euler angle triplets
@@ -124,22 +124,16 @@ class TestAngPlugin:
         indirect=["angfile_tsl"],
     )
     def test_load_ang_tsl(
-        self,
-        angfile_tsl,
-        map_shape,
-        step_sizes,
-        phase_id,
-        n_unknown_columns,
-        example_rot,
+        self, angfile_tsl, map_shape, step_sizes, phase_id, example_rot,
     ):
-        cm = load(angfile_tsl)
+        xmap = load(angfile_tsl)
 
         # Fraction of non-indexed points
         non_indexed_fraction = int(np.prod(map_shape) * 0.1)
-        assert non_indexed_fraction == np.sum(~cm.is_indexed)
+        assert non_indexed_fraction == np.sum(~xmap.is_indexed)
 
         # Properties
-        assert list(cm.prop.keys()) == [
+        assert list(xmap.prop.keys()) == [
             "iq",
             "ci",
             "unknown1",
@@ -153,38 +147,38 @@ class TestAngPlugin:
         # Coordinates
         ny, nx = map_shape
         dy, dx = step_sizes
-        assert np.allclose(cm.x, np.tile(np.arange(nx) * dx, ny))
-        assert np.allclose(cm.y, np.sort(np.tile(np.arange(ny) * dy, nx)))
+        assert np.allclose(xmap.x, np.tile(np.arange(nx) * dx, ny))
+        assert np.allclose(xmap.y, np.sort(np.tile(np.arange(ny) * dy, nx)))
 
         # Map shape and size
-        assert cm.shape == map_shape
-        assert cm.size == np.prod(map_shape)
+        assert xmap.shape == map_shape
+        assert xmap.size == np.prod(map_shape)
 
         # Attributes are within expected ranges or have a certain value
-        assert cm.prop["ci"].max() <= 1
-        assert cm["indexed"].fit.max() <= 3
-        assert all(cm["not_indexed"].fit == 180)
-        assert all(cm["not_indexed"].ci == -1)
+        assert xmap.prop["ci"].max() <= 1
+        assert xmap["indexed"].fit.max() <= 3
+        assert all(xmap["not_indexed"].fit == 180)
+        assert all(xmap["not_indexed"].ci == -1)
 
         # Phase IDs (accounting for non-indexed points)
-        phase_id[cm["not_indexed"].id] = -1
-        assert np.allclose(cm.phase_id, phase_id)
+        phase_id[xmap["not_indexed"].id] = -1
+        assert np.allclose(xmap.phase_id, phase_id)
 
         # Rotations
-        rot_unique = np.unique(cm["indexed"].rotations.to_euler(), axis=0)
+        rot_unique = np.unique(xmap["indexed"].rotations.to_euler(), axis=0)
         assert np.allclose(
             np.sort(rot_unique, axis=0), np.sort(example_rot, axis=0), atol=1e-5
         )
         assert np.allclose(
-            cm["not_indexed"].rotations.to_euler()[0],
+            xmap["not_indexed"].rotations.to_euler()[0],
             np.array([np.pi, 0, np.pi]),
             atol=1e-5,
         )
 
         # Phases
-        assert cm.phases.size == 2  # Including non-indexed
-        assert cm.phases.ids == [-1, 0]
-        phase = cm.phases[0]
+        assert xmap.phases.size == 2  # Including non-indexed
+        assert xmap.phases.ids == [-1, 0]
+        phase = xmap.phases[0]
         assert phase.name == "Aluminum"
         assert phase.point_group.name == "432"
 
@@ -514,3 +508,168 @@ class TestAngPlugin:
         assert point_groups == expected_point_groups
         assert np.allclose(lattice_constants, expected_lattice_constants)
         assert np.allclose(ids, expected_phase_id)
+
+
+class TestAngWriter:
+    def test_write_read_loop(self, crystal_map, tmp_path):
+        fname = tmp_path / "test_write_read_loop.ang"
+        save(filename=fname, object2write=crystal_map)
+        xmap_reload = load(filename=fname)
+
+        assert np.allclose(
+            xmap_reload.rotations.to_euler(), crystal_map.rotations.to_euler(),
+        )
+        assert np.allclose(xmap_reload.phase_id - 1, crystal_map.phase_id)
+
+    @pytest.mark.parametrize(
+        "crystal_map_input, desired_shape, desired_step_sizes",
+        [
+            (((1, 4, 3), (1, 2, 3), 1, [0]), (4, 3), (2, 3)),
+            (((1, 1, 3), (1, 1, 1), 1, [0]), (1, 3), (1, 1)),
+            (((1, 1, 6), (1, 1, 3.14), 1, [0]), (1, 6), (1, 3.14)),
+        ],
+        indirect=["crystal_map_input"],
+    )
+    def test_get_nrows_ncols_step_sizes(
+        self, crystal_map_input, desired_shape, desired_step_sizes
+    ):
+        xmap = CrystalMap(**crystal_map_input)
+        nrows, ncols, dy, dx = _get_nrows_ncols_step_sizes(xmap)
+
+        assert (nrows, ncols) == desired_shape
+        assert (dy, dx) == desired_step_sizes
+
+    @pytest.mark.parametrize(
+        "max_value, decimals, expected_width",
+        [(3.14, 2, 5), (3.1415, 2, 5), (3.141592, 6, 9)],
+    )
+    def test_get_column_width(self, max_value, decimals, expected_width):
+        assert _get_column_width(max_value, decimals) == expected_width
+
+    @pytest.mark.parametrize(
+        "extra_prop", ["a", ["abc", "iq"], ["scores", "simulation_indices"]]
+    )
+    def test_extra_prop(self, crystal_map, tmp_path, extra_prop):
+        fname = tmp_path / "test_extra_prop.ang"
+        desired_arrays = []
+        n_points = crystal_map.size
+        for i, name in enumerate(extra_prop):
+            new_array = np.arange(n_points) * i
+            crystal_map.prop[name] = new_array
+            desired_arrays.append(new_array)
+
+        save(fname, crystal_map, extra_prop=extra_prop)
+        xmap_reload = load(fname)
+
+        for name in extra_prop:
+            assert np.allclose(xmap_reload.prop[name], crystal_map.prop[name])
+
+    def test_non_indexed_points(self, crystal_map, tmp_path):
+        crystal_map[2].phase_id = -1
+        fname = tmp_path / "test_non_indexed_points.ang"
+        save(fname, crystal_map)
+
+        xmap_reload = load(fname)
+
+        crystal_map.phases[0].name = "phase1"
+        assert xmap_reload.phases.names == crystal_map.phases.names
+
+        new_phase_ids = xmap_reload.phase_id
+        new_phase_ids[xmap_reload.is_indexed] -= 1
+        assert np.allclose(new_phase_ids, crystal_map.phase_id)
+
+    def test_points_not_in_data(self, crystal_map, tmp_path):
+        crystal_map.prop["iq"] = np.ones(crystal_map.size)
+        crystal_map[2].iq = 0
+        xmap2 = crystal_map[crystal_map.iq == 1]
+        fname = tmp_path / "test_points_not_in_data.ang"
+        save(fname, xmap2)
+
+        xmap_reload = load(fname)
+        assert not xmap2.is_in_data.all()
+        assert np.allclose(xmap2.is_in_data, xmap_reload.is_indexed)
+
+    @pytest.mark.parametrize(
+        "extra_phase_names", ["ni", ["ferrite", "austenite"], ["si", "al", "cu"]],
+    )
+    def test_extra_phases(self, crystal_map, tmp_path, extra_phase_names):
+        crystal_map.phases.add_not_indexed()
+        for i, name in enumerate(extra_phase_names):
+            crystal_map.phases.add(Phase(name=name))
+            crystal_map[i].phase_id = crystal_map.phases.id_from_name(name)
+        fname = tmp_path / "test_extra_phases.ang"
+        save(fname, crystal_map)
+        xmap_reload = load(fname)
+
+        crystal_map.phases[0].name = "phase1"
+        assert np.allclose(xmap_reload.phase_id - 1, crystal_map.phase_id)
+
+        pl = crystal_map.phases
+        del pl[-1]
+        assert xmap_reload.phases.names == pl.names
+
+    @pytest.mark.parametrize("point_group", ["432", "121", "222"])
+    def test_point_group_aliases(self, crystal_map, tmp_path, point_group):
+        crystal_map.phases[0].point_group = point_group
+        fname = tmp_path / "test_point_group_aliases.ang"
+        save(fname, crystal_map)
+        xmap_reload = load(fname)
+
+        phase_ids = xmap_reload.phases.ids
+        assert xmap_reload.phases[phase_ids[0]].point_group.name == point_group
+
+    @pytest.mark.parametrize(
+        "crystal_map_input",
+        [((1, 1, 5), (1, 1, 2), 1, [0])],
+        indirect=["crystal_map_input"],
+    )
+    def test_1d_map(self, crystal_map_input, tmp_path):
+        xmap = CrystalMap(**crystal_map_input)
+        assert xmap.ndim == 1
+        fname = tmp_path / "test_1d_map.ang"
+        save(fname, xmap)
+
+        xmap_reload = load(fname)
+
+        assert xmap_reload.ndim == 1
+        assert np.allclose(xmap.rotations.to_euler(), xmap_reload.rotations.to_euler())
+
+    @pytest.mark.parametrize(
+        "crystal_map_input",
+        [((3, 3, 3), (1, 2, 3), 1, [0])],
+        indirect=["crystal_map_input"],
+    )
+    def test_3d_map_raises(self, crystal_map_input, tmp_path):
+        xmap = CrystalMap(**crystal_map_input)
+        fname = tmp_path / "test_3d_raises.ang"
+        with pytest.raises(ValueError, match="Writing a 3D dataset to an .ang file"):
+            save(fname, xmap)
+
+    @pytest.mark.parametrize(
+        "crystal_map_input, index",
+        [
+            (((1, 4, 3), (1, 2, 3), 5, [0]), 0),
+            (((1, 4, 3), (1, 2, 3), 5, [0]), 1),
+            (((1, 4, 3), (1, 2, 3), 5, [0]), 4),
+        ],
+        indirect=["crystal_map_input"],
+    )
+    def test_write_data_layer_i(self, crystal_map_input, tmp_path, index):
+        xmap = CrystalMap(**crystal_map_input)
+        xmap.prop["ci"] = np.arange(xmap.size * xmap.rotations_per_point).reshape(
+            (xmap.size, xmap.rotations_per_point)
+        )
+        xmap.prop["iq"] = np.arange(xmap.size)
+        extra_prop = "iq_times_ci"
+        xmap.prop[extra_prop] = xmap.ci * xmap.iq[:, np.newaxis]
+        fname = tmp_path / "test_write_data_layer_i.ang"
+        save(fname, xmap, index=index, extra_prop=[extra_prop, "iq"])
+
+        xmap_reload = load(fname)
+
+        assert np.allclose(
+            xmap.rotations[:, index].to_euler(), xmap_reload.rotations.to_euler()
+        )
+        assert np.allclose(xmap_reload.iq, np.zeros(xmap.size))
+        assert np.allclose(xmap_reload.ci, xmap.ci[:, index])
+        assert np.allclose(xmap_reload.iq_times_ci, xmap.iq_times_ci[:, index])
